#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

constexpr byte Char_Remain = Char_Man;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 0 * VramStep, "SC");   
    // PrintS(Vram + 7 * VramStep, "HI-SCORE");
    PrintS(Vram + 9 * VramStep, "ST");
    PrintByteNumber2(Vram + 11 * VramStep, CurrentStage + 1);
    PrintC(Vram+ 14 * VramStep, 'T');
    {
        ptr<byte> pVram;
        pVram = Vram + 20 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                pVram = Put2C(pVram, Char_Remain);
                *pVram = Char_Space; ++pVram;
                pVram = PrintC(pVram, i + 0x30);
            }
            else {
                do {
                    pVram = Put2C(pVram, Char_Remain);
                    --i;
                } while (i > 0);
            }
        }
    }
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + 2 * VramStep, Score);
    PrintC(pVram, '0');

    // pVram = PrintNumber5(Vram + 9 * VramStep, HiScore);
    // PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + 15 * VramStep, StageTime);
}

void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 4 + 8 * VramStep, "GAME OVER");
    PresendVram();
}

void PrintTimeUp() 
{
    PrintS(Vram + VramRowSize * 4 + 9 * VramStep, "TIME UP");
    PresendVram();
}

void Title()
{
    static const byte[] TitleBytes = {
        //	C
        0x00, 0x0e, 0x05, 0x0b, 
        0x0c, 0x03, 0x00, 0x00, 
        0x04, 0x0b, 0x00, 0x0a, 
        0x00, 0x04, 0x05, 0x01, 
        //	R
        0x0c, 0x07, 0x05, 0x0b, 
        0x0c, 0x03, 0x08, 0x0f, 
        0x0c, 0x07, 0x0f, 0x02, 
        0x04, 0x01, 0x04, 0x05, 
        //	A
        0x00, 0x0e, 0x0d, 0x02, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x07, 0x05, 0x0f, 
        0x04, 0x01, 0x00, 0x05, 
        //	C
        0x00, 0x0e, 0x05, 0x0b, 
        0x0c, 0x03, 0x00, 0x00, 
        0x04, 0x0b, 0x00, 0x0a, 
        0x00, 0x04, 0x05, 0x01, 
        //	K
        0x0c, 0x03, 0x08, 0x07, 
        0x0c, 0x0b, 0x07, 0x00, 
        0x0c, 0x03, 0x0d, 0x02, 
        0x04, 0x01, 0x00, 0x05, 
        //	Y
        0x0c, 0x03, 0x0c, 0x03, 
        0x04, 0x0b, 0x0e, 0x01, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x04, 0x01, 0x00, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 1 + 0 * VramStep;
        p = TitleBytes;
        repeat (6) {
            repeat (2) {
                repeat (4) {
                    pVram = PutU(pVram, *p);
                    ++p;
                }
                pVram -= 4 * VramStep;
                repeat (4) {
                    pVram = PutL(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 2;
        }
        PrintS(Vram + VramRowSize * 3 + 18 * VramStep, "MINI");
    }

    PrintS(Vram + VramRowSize * 5 + 5 * VramStep, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 6 + 5 * VramStep, "OR GRPH KEY");
    PrintS(Vram + VramRowSize * 7 + 12 * VramStep, "INUFUTO 2023");

    PresendVram();
}